import ArrowUpwardIcon from '@mui/icons-material/ArrowUpward'
import NotificationsActiveIcon from '@mui/icons-material/NotificationsActive'
import {
  Accordion,
  AccordionDetails,
  AccordionSummary,
  Button,
  InputAdornment,
  Paper,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableHead,
  TableRow,
  TableSortLabel,
  TextField,
  Typography,
} from '@mui/material'
import { Box, styled } from '@mui/system'
import { useState } from 'react'
import SearchIcon from '@mui/icons-material/Search'

import Loading from 'components/loading'
import Page from 'components/page'
import { differenceInDays, format, isBefore } from 'date-fns'
import { Guest, Role } from 'interfaces'
import { Trans, useTranslation } from 'react-i18next'
import { Link } from 'react-router-dom'
import { getRoleName, getRoleOuName } from 'utils'
import SponsorGuestButtons from '../../components/sponsorGuestButtons'

interface GuestProps {
  persons: Guest[]
}

interface PersonLineProps {
  person: Guest
  role: Role
}

interface StatusProps {
  person: Guest
  role: Role
}

interface GuestTableProps {
  guests: Guest[]
  emptyText: string
  marginWidth: number
}

interface FrontPageProps {
  guests: Guest[]
  loading: boolean
}

type SortDirection = 'desc' | 'asc'
type SortField = 'name' | 'role' | 'status' | 'endDate' | 'department'

interface GuestRole {
  guest: Guest
  role: Role
}

const StyledTableRow = styled(TableRow)({
  borderTop: '0',
  borderLeft: '0',
  borderRight: '0',
  borderBottom: '2px solid',
  borderColor: 'black',
  borderRadius: '0',
})

const StyledAccordion = styled(Accordion)({
  borderStyle: 'none',
  boxShadow: 'none',
  margin: 'auto',
})

const StyledAccordionSummary = styled(AccordionSummary)({
  borderStyle: 'solid',
  borderColor: 'black',
  borderWidth: '0.125rem',
  borderRadius: '1%',
})

const StyledTableHeadCell = styled(TableCell)({
  fontWeight: 'bold',
})

const StyledTableHead = styled(TableHead)(({ theme }) => ({
  borderTop: '0',
  borderLeft: '0',
  borderRight: '0',
  borderBottom: '0.1875rem solid',
  borderColor: theme.palette.secondary.main,
  borderRadius: '0',
}))

const calculateStatus = (person: Guest, role: Role): [string, number] => {
  const today = new Date()
  today.setHours(0, 0, 0, 0)
  let status = ''
  const days = differenceInDays(role.end_date, today)

  if (!person.registered) {
    status = 'waitingForGuest'
    if (person.invitation_status === 'invalidEmail') {
      status = 'invalidEmail'
    } else if (person.invitation_status !== 'active') {
      status = 'invitationExpired'
    }
  } else if (person.registered && !person.verified) {
    status = 'waitingForSponsor'
  } else if (person.registered && person.verified) {
    if (isBefore(role.end_date, today)) {
      status = 'expired'
    } else if (days <= 7) {
      status = 'expiring'
    } else {
      status = 'active'
    }
  }
  return [status, days]
}

const Status = ({ person, role }: StatusProps) => {
  const { t } = useTranslation('common')
  const [status, days] = calculateStatus(person, role)

  switch (status) {
    case 'active':
      return (
        <TableCell sx={{ color: 'success.main' }} align="left">
          {t('statusText.active')}
        </TableCell>
      )
    case 'expired':
      return (
        <TableCell sx={{ color: 'error.main' }} align="left">
          {t('statusText.expired')}
        </TableCell>
      )
    case 'waitingForGuest':
      return (
        <TableCell sx={{ color: 'blue' }} align="left">
          {t('statusText.waitingForGuest')}
        </TableCell>
      )
    case 'waitingForSponsor':
      return (
        <TableCell sx={{ color: 'blue' }} align="left">
          {t('statusText.waitingForSponsor')}
        </TableCell>
      )
    case 'invitationExpired':
      return (
        <TableCell sx={{ color: 'error.main' }} align="left">
          {t('statusText.invitationExpired')}
        </TableCell>
      )
    case 'expiring':
      return (
        <TableCell sx={{ color: 'blue' }} align="left">
          <Trans t={t} i18nKey="statusText.expiring" count={days} />
        </TableCell>
      )
    case 'invalidEmail':
      return (
        <TableCell sx={{ color: 'error.main' }} align="left">
          <Trans t={t} i18nKey="statusText.invalidEmail" count={days} />
        </TableCell>
      )
    default:
      return (
        <TableCell sx={{ color: 'error.main' }} align="left">
          {t('statusText.waiting')}
        </TableCell>
      )
  }
}

function createGuestsRoles(guests: Guest[]): GuestRole[] {
  const guestRoles: GuestRole[] = []
  guests.forEach((guest) =>
    guest.roles.forEach((role) => {
      guestRoles.push({ guest, role })
    })
  )
  return guestRoles
}

function sortByName(
  guestA: Guest,
  guestB: Guest,
  direction: SortDirection
): number {
  const firstA = guestA.first.toLowerCase()
  const firstB = guestB.first.toLowerCase()
  if (firstA < firstB) {
    return direction === 'asc' ? -1 : 1
  }
  if (firstA > firstB) {
    return direction === 'asc' ? 1 : -1
  }
  return 0
}

function sortGuestsByName(guests: Guest[], direction: SortDirection) {
  return createGuestsRoles(guests).sort((a, b) =>
    sortByName(a.guest, b.guest, direction)
  )
}

function sortGuestRolesByEndDate(
  guestRole: GuestRole[],
  direction: SortDirection
): GuestRole[] {
  return guestRole.sort((a, b) => {
    const aEndDate = a.role.end_date.getTime()
    const bEndDate = b.role.end_date.getTime()

    if (aEndDate < bEndDate) {
      return direction === 'asc' ? -1 : 1
    }
    if (aEndDate > bEndDate) {
      return direction === 'asc' ? 1 : -1
    }
    // Secondary sort on guest name, always asc
    return sortByName(a.guest, b.guest, 'asc')
  })
}

function sortByStatus(guests: Guest[], direction: SortDirection): GuestRole[] {
  let activeRoleAndPerson: { guest: Guest; role: Role }[] = []
  let expiringRoleAndPerson: { guest: Guest; role: Role }[] = []
  let expiredRoleAndPerson: { guest: Guest; role: Role }[] = []

  guests.forEach((guest) =>
    guest.roles.forEach((role) => {
      const [status] = calculateStatus(guest, role)
      if (status === 'active') {
        activeRoleAndPerson.push({ guest, role })
      } else if (status === 'expiring') {
        expiringRoleAndPerson.push({ guest, role })
      } else {
        expiredRoleAndPerson.push({ guest, role })
      }
    })
  )

  // Always use 'asc' for secondary sorting
  activeRoleAndPerson = sortGuestRolesByEndDate(activeRoleAndPerson, 'asc')
  expiringRoleAndPerson = sortGuestRolesByEndDate(expiringRoleAndPerson, 'asc')
  expiredRoleAndPerson = sortGuestRolesByEndDate(expiredRoleAndPerson, 'asc')

  if (direction === 'asc') {
    return expiringRoleAndPerson
      .concat(activeRoleAndPerson)
      .concat(expiredRoleAndPerson)
  }
  return expiredRoleAndPerson
    .concat(activeRoleAndPerson)
    .concat(expiringRoleAndPerson)
}

function sortGuestsByRoleName(guests: Guest[], direction: SortDirection) {
  return createGuestsRoles(guests).sort((a, b) => {
    const aRoleName = getRoleName(a.role).toLowerCase()
    const bRoleName = getRoleName(b.role).toLowerCase()
    if (aRoleName < bRoleName) {
      return direction === 'asc' ? -1 : 1
    }
    if (aRoleName > bRoleName) {
      return direction === 'asc' ? 1 : -1
    }
    // Secondary sort on guest name, always asc
    return sortByName(a.guest, b.guest, 'asc')
  })
}

function sortGuestsByDepartmentName(guests: Guest[], direction: SortDirection) {
  return createGuestsRoles(guests).sort((a, b) => {
    const aRoleOUName = getRoleOuName(a.role).toLowerCase()
    const bRoleOUName = getRoleOuName(b.role).toLowerCase()
    if (aRoleOUName < bRoleOUName) {
      return direction === 'asc' ? -1 : 1
    }
    if (aRoleOUName > bRoleOUName) {
      return direction === 'asc' ? 1 : -1
    }
    // Secondary sort on guest name, always asc
    return sortByName(a.guest, b.guest, 'asc')
  })
}

function sortGuestsAndRoles(
  guests: Guest[],
  sortField: SortField,
  direction: SortDirection
): { guest: Guest; role: Role }[] {
  switch (sortField) {
    case 'department':
      return sortGuestsByDepartmentName(guests, direction)
    case 'endDate':
      return sortGuestRolesByEndDate(createGuestsRoles(guests), direction)
    case 'name':
      return sortGuestsByName(guests, direction)
    case 'role':
      return sortGuestsByRoleName(guests, direction)
    case 'status':
      return sortByStatus(guests, direction)
    default:
      // Fallback to original sort
      return createGuestsRoles(guests)
  }
}

const PersonLine = ({ person, role }: PersonLineProps) => {
  const [t] = useTranslation(['common'])

  return (
    <StyledTableRow>
      <TableCell component="th" scope="row">
        {`${person.first} ${person.last}`}
      </TableCell>
      <TableCell align="left">{getRoleName(role)}</TableCell>
      <Status person={person} role={role} />
      <TableCell align="left"> {format(role.end_date, 'yyyy-MM-dd')}</TableCell>
      <TableCell align="left">{getRoleOuName(role)}</TableCell>
      <TableCell align="left">
        <Button
          variant="contained"
          color="secondary"
          component={Link}
          to={`/sponsor/guest/${person.pid}`}
        >
          {t('common:details')}
        </Button>
      </TableCell>
    </StyledTableRow>
  )
}

const GuestTable = ({ guests, emptyText, marginWidth }: GuestTableProps) => {
  const { t } = useTranslation('common')

  const [direction, setDirection] = useState<SortDirection>('asc')
  const [orderBy, setOrderBy] = useState<SortField>('status')

  const handleRequestSort =
    (sortBy: SortField) => (event: React.MouseEvent<unknown>) => {
      event.preventDefault()
      const isAsc = orderBy === sortBy && direction === 'asc'
      setDirection(isAsc ? 'desc' : 'asc')
      setOrderBy(sortBy)
    }

  return (
    <TableContainer
      component={Paper}
      sx={{
        boxShadow: 'none',
        borderRadius: '0rem',
        minWidth: marginWidth,
      }}
    >
      <Table sx={{ minWidth: marginWidth }} aria-label="simple table">
        <StyledTableHead>
          <TableRow>
            <StyledTableHeadCell>
              <TableSortLabel
                key="name"
                active={orderBy === 'name'}
                direction={orderBy === 'name' ? direction : 'asc'}
                onClick={handleRequestSort('name')}
              >
                {t('common:name')}
              </TableSortLabel>
            </StyledTableHeadCell>
            <StyledTableHeadCell>
              <TableSortLabel
                key="role"
                active={orderBy === 'role'}
                direction={orderBy === 'role' ? direction : 'asc'}
                onClick={handleRequestSort('role')}
              >
                {t('common:role')}
              </TableSortLabel>
            </StyledTableHeadCell>

            <StyledTableHeadCell>
              <TableSortLabel
                key="status"
                active={orderBy === 'status'}
                direction={orderBy === 'status' ? direction : 'asc'}
                onClick={handleRequestSort('status')}
              >
                {t('common:status')}
              </TableSortLabel>
            </StyledTableHeadCell>
            <StyledTableHeadCell>
              <TableSortLabel
                key="endDate"
                active={orderBy === 'endDate'}
                direction={orderBy === 'endDate' ? direction : 'asc'}
                onClick={handleRequestSort('endDate')}
              >
                {t('common:endDate')}
              </TableSortLabel>
            </StyledTableHeadCell>
            <StyledTableHeadCell>
              <TableSortLabel
                key="department"
                active={orderBy === 'department'}
                direction={orderBy === 'department' ? direction : 'asc'}
                onClick={handleRequestSort('department')}
              >
                {t('common:department')}
              </TableSortLabel>
            </StyledTableHeadCell>
            <StyledTableHeadCell />
          </TableRow>
        </StyledTableHead>
        <TableBody>
          {guests.length > 0 ? (
            sortGuestsAndRoles(guests, orderBy, direction).map((personRole) => (
              <PersonLine
                key={`${personRole.guest.first} ${personRole.guest.last} ${personRole.role.id}`}
                role={personRole.role}
                person={personRole.guest}
              />
            ))
          ) : (
            <StyledTableRow>
              <TableCell>{emptyText}</TableCell>
            </StyledTableRow>
          )}
        </TableBody>
      </Table>
    </TableContainer>
  )
}

const InvitedGuests = ({ persons }: GuestProps) => {
  const [activeExpanded, setActiveExpanded] = useState(false)

  // Show guests that have not responded to the invite yet
  let guests = persons.length > 0 ? persons : []

  if (guests.length > 0) {
    guests = guests.filter((person) => !person.registered)
  }
  const [t] = useTranslation(['common'])
  return (
    <StyledAccordion
      expanded={activeExpanded}
      onChange={() => {
        setActiveExpanded(!activeExpanded)
      }}
    >
      <StyledAccordionSummary
        expandIcon={<ArrowUpwardIcon sx={{ fill: 'black' }} />}
      >
        <Typography variant="h2">
          {`${t('common:sentInvitations')} (${guests.length})`}
        </Typography>
      </StyledAccordionSummary>
      <AccordionDetails>
        <Typography
          variant="body1"
          sx={{ marginTop: '1rem', marginBottom: '1rem' }}
        >
          {t('common:sentInvitationsDescription')}
        </Typography>
        <GuestTable
          guests={guests}
          emptyText={t('common:noInvitations')}
          marginWidth={650}
        />
      </AccordionDetails>
    </StyledAccordion>
  )
}

const ActiveGuests = ({ persons }: GuestProps) => {
  const [activeExpanded, setActiveExpanded] = useState(false)
  const [searchHasInput, setSearchHasInput] = useState(false)
  const [searchGuests, setSearchGuests] = useState<Guest[]>([])

  // Show all verified guests
  let guests = persons.length > 0 ? persons : []
  if (guests.length > 0) {
    guests = guests.filter((person) => person.verified)
  }
  const [t] = useTranslation(['common'])

  const getSponsorGuests = (event: React.ChangeEvent<HTMLInputElement>) => {
    if (event.target.value) {
      setSearchHasInput(true)
      const guestSearch: Guest[] = guests.filter((guest) =>
        `${guest.first.toLowerCase()} ${guest.last.toLowerCase()}`.includes(
          event.target.value.toLowerCase()
        )
      )
      setSearchGuests(guestSearch)
    } else {
      setSearchHasInput(false)
      setSearchGuests([])
    }
  }
  return (
    <StyledAccordion
      expanded={activeExpanded}
      onChange={() => {
        setActiveExpanded(!activeExpanded)
      }}
    >
      <StyledAccordionSummary
        expandIcon={<ArrowUpwardIcon sx={{ fill: 'black' }} />}
      >
        <Typography variant="h2">
          {`${t('common:activeGuests')} (${guests.length})`}
        </Typography>
      </StyledAccordionSummary>
      <AccordionDetails>
        <Typography
          variant="body1"
          sx={{ marginTop: '1rem', marginBottom: '1rem' }}
        >
          {t('common:activeGuestsDescription')}
        </Typography>
        <Box
          sx={{
            marginBottom: '1rem',
          }}
        >
          <TextField
            variant="standard"
            InputProps={{
              endAdornment: (
                <InputAdornment position="end">
                  <SearchIcon />
                </InputAdornment>
              ),
            }}
            placeholder={t('placeholder')}
            onChange={getSponsorGuests}
          />
          {!searchHasInput ? (
            <GuestTable
              guests={guests}
              emptyText={t('common:noActiveGuests')}
              marginWidth={1000}
            />
          ) : (
            <GuestTable
              guests={searchGuests}
              emptyText={t('common:foundNoGuests')}
              marginWidth={1000}
            />
          )}
        </Box>
      </AccordionDetails>
    </StyledAccordion>
  )
}

const WaitingGuests = ({ persons }: GuestProps) => {
  const [waitingExpanded, setWaitingExpanded] = useState(false)

  // Show guests that have completed the registration but are not verified yet
  let guests = persons.length > 0 ? persons : []
  if (guests.length > 0) {
    guests = guests.filter((person) => person.registered && !person.verified)
  }
  const [t] = useTranslation(['common'])

  return (
    <StyledAccordion
      expanded={waitingExpanded}
      onChange={() => {
        setWaitingExpanded(!waitingExpanded)
      }}
    >
      <StyledAccordionSummary
        expandIcon={<ArrowUpwardIcon sx={{ fill: 'black' }} />}
      >
        <Typography variant="h2">
          {`${t('common:waitingGuests')} (${guests.length})`}{' '}
          {guests.length > 0 && (
            <NotificationsActiveIcon
              sx={{ color: 'error.main', fontSize: '1.625rem' }}
            />
          )}
        </Typography>
      </StyledAccordionSummary>
      <AccordionDetails>
        <Typography
          variant="body1"
          sx={{ marginTop: '1rem', marginBottom: '1rem' }}
        >
          <Trans i18nKey="common:waitingGuestsDescription">
            Her godkjenner du gjester som har <b>registrert seg manuelt</b>.
            Gjester som har FEIDE-bruker trenger ikke godkjenning.
          </Trans>
        </Typography>
        <GuestTable
          guests={guests}
          emptyText={t('common:noWaitingGuests')}
          marginWidth={650}
        />
      </AccordionDetails>
    </StyledAccordion>
  )
}

function FrontPage({ guests, loading }: FrontPageProps) {
  return (
    <Page pageWidth>
      <SponsorGuestButtons yourGuestsActive />

      {loading ? (
        <Loading />
      ) : (
        <>
          <InvitedGuests persons={guests} />
          <br />
          <WaitingGuests persons={guests} />
          <br />
          <ActiveGuests persons={guests} />
        </>
      )}
    </Page>
  )
}

export default FrontPage
